package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.EartagUtil;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.UploadImage;
import com.huimv.farm.damsubsidy.entity.BaseAnimal;
import com.huimv.farm.damsubsidy.entity.BillIsolated;
import com.huimv.farm.damsubsidy.entity.BillLandingInspection;
import com.huimv.farm.damsubsidy.entity.EartagResult;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillIsolatedMapper;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.service.IBillIsolatedService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillIsolatedServiceImpl extends ServiceImpl<BillIsolatedMapper, BillIsolated> implements IBillIsolatedService {

    @Resource
    private BillLandingInspectionMapper landingInspectionMapper;

    @Resource
    private BaseAnimalMapper baseAnimalMapper;

    @Autowired
    private UploadImage uploadImage;

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public synchronized Result addIsolation(HttpServletRequest httpServletRequest, MultipartFile observeImage, String inspectionNum, String isolatedPosition, String strIsolated, String dayIsolated) throws ParseException, IOException {
        int dayIsolate = Integer.parseInt(dayIsolated);
        if (com.baomidou.mybatisplus.core.toolkit.StringUtils.isBlank(inspectionNum) || inspectionNum.length()!=10){
            return new Result(10001,"检疫证号不符合规范！",false);
        }
        if (dayIsolate > 99 || dayIsolate < 15 ){
            return new Result(10001,"隔离天数请在15-99天！",false);
        }
        Integer integer = this.count(new QueryWrapper<BillIsolated>().eq("inspection_num", inspectionNum));
        if (integer != 0){
            return new Result(10001,"检疫证号重复！",false);
        }
        BillIsolated billIsolated = new BillIsolated();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date startTime = dateFormat.parse(strIsolated);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        calendar.add(Calendar.DATE, dayIsolate);
        Date endTime = calendar.getTime();
        String userName = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum).ne("inspection_sch",5)).getCargoOwner();
        billIsolated.setInspectionName(userName+"的隔离信息");
        billIsolated.setInspectionNum(inspectionNum);
        billIsolated.setIsolatedPosition(isolatedPosition);
        billIsolated.setStrIsolated(startTime);
        billIsolated.setDayIsolated(Integer.parseInt(dayIsolated));
        billIsolated.setEndIsolated(endTime);
        billIsolated.setCreateTime(DateTime.now());
        billIsolated.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));

        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum).ne("inspection_sch", 5));
        String eartags = billLandingInspection.getEartags();
        EartagResult eartagCollection = EartagUtil.getEartagCollection(eartags);
        if (!eartagCollection.getSuccess()) {
            return new Result(10001,eartagCollection.getMsg(),false);
        }

        List<String> eartagList = eartagCollection.getEartagList();
        for (String eartagNo : eartagList) {
            Integer num = baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartagNo));
            if (num != 0){
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"耳标号"+eartagNo+"已存在",false);
            }
            BaseAnimal baseAnimal = new BaseAnimal();
            baseAnimal.setEaratgNo(eartagNo);
            baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
            baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
            baseAnimal.setBelongToUser(billLandingInspection.getCreateUser());
            baseAnimal.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            baseAnimal.setCreateTime(new Date());
            baseAnimalMapper.insert(baseAnimal);
        }
        String imageCom = uploadImage.getImageCom(observeImage);
        billLandingInspection.setObservePicUrl(imageCom);
        billLandingInspection.setInspectionSch(3);
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setReviewedTime(DateTime.now());
        landingInspectionMapper.updateById(billLandingInspection);
        this.save(billIsolated);
        return Result.SUCCESS();

    }

    public String getUrl(String content, MultipartFile image) throws IOException {
        if (ObjectUtil.isNotEmpty(image)){
            if ( !"".equals(content)){
                content = content +","+uploadImage.getImageCom(image);
            }else {
                content =   content +uploadImage.getImageCom(image);
            }
        }
        return content;
    }

    @Override
    public Result listIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String type = paramsMap.get("type");
        Page<BillIsolated> page = new Page(Integer.parseInt(pageNum),Integer.parseInt(pageSize));
        if (type.equals("1")){
            return new Result(ResultCode.SUCCESS,this.page(page,new QueryWrapper<BillIsolated>().ge("str_isolated",LocalDate.now()).le("end_isolated",LocalDate.now())));
        }else {
            return new Result(ResultCode.SUCCESS,this.page(page,new QueryWrapper<BillIsolated>().lt("end_isolated",LocalDate.now())));
        }
    }

    @Override
    public Result delayedIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String day = paramsMap.get("day");
        if (day.contains(".")){
            return new Result(10001,"请输入整数",false);
        }
        BillIsolated billIsolated = this.getById(id);
        billIsolated.setDayIsolated(billIsolated.getDayIsolated()+Integer.parseInt(day));
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(billIsolated.getEndIsolated());
        calendar.add(Calendar.DATE, Integer.valueOf(day));
        Date endTime = calendar.getTime();
        billIsolated.setUpdateUser(TokenSign.getUserName(httpServletRequest));
        billIsolated.setUpdateTime(DateTime.now());
        billIsolated.setEndIsolated(endTime);
        this.updateById(billIsolated);
        return Result.SUCCESS();
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public Result cancelIsolation(HttpServletRequest httpServletRequest,MultipartFile isolatedImage1,MultipartFile isolatedImage2,MultipartFile isolatedImage3 ,MultipartFile observeSecureImage, String id, String isDead, String deaths) throws IOException {
        String content = "";
        content = getUrl(content,isolatedImage1);
        content = getUrl(content,isolatedImage2);
        content = getUrl(content,isolatedImage3);
        String observeImage = uploadImage.getImageCom(observeSecureImage);
        BillIsolated isolation = this.getById(id);
        isolation.setObserveSecurePicUrl(observeImage);
        isolation.setIsolatedPicUrl(content);
        isolation.setIsolatedSch(2);
        if ("1".equals(isDead)){
            isolation.setIsDead(1);
            isolation.setDeaths(deaths);
            EartagResult eartagCollection = EartagUtil.getEartagCollection(deaths);
            if (!eartagCollection.getSuccess()){
                return new Result(10001,eartagCollection.getMsg(),false);
            }
            List<String> eartagList = eartagCollection.getEartagList();
            for (String number : eartagList) {
                BaseAnimal animal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", number));
                if (ObjectUtil.isEmpty(animal)){
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return new Result(10001,"耳标号"+number+"不存在",false);
                }
                animal.setIsDie(1);
                if (baseAnimalMapper.updateById(animal) != 1) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return new Result(10001,"操作失败",false);
                }
            }
        }else {
            isolation.setIsDead(0);
        }
        List<BaseAnimal> baseAnimals = baseAnimalMapper.selectList(new QueryWrapper<BaseAnimal>().eq("inspection_num", isolation.getInspectionNum()));
        for (BaseAnimal baseAnimal : baseAnimals) {
            baseAnimal.setAnimalStatus(1);
            if (baseAnimalMapper.updateById(baseAnimal) != 1) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"操作失败",false);
            }
        }
        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>()
                .eq("inspection_num", isolation.getInspectionNum()).eq("inspection_sch",3).eq("inspection_type",0));
        billLandingInspection.setReviewedTime(DateTime.now());
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setInspectionSch(4);
        landingInspectionMapper.updateById(billLandingInspection);
        isolation.setUpdateTime(DateTime.now());
        isolation.setUpdateUser(TokenSign.getUserName(httpServletRequest));
        this.updateById(isolation);
        return Result.SUCCESS();
    }

}
