package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.*;
import com.huimv.farm.damsubsidy.entity.*;
import com.huimv.farm.damsubsidy.entity.BillLandingInspection;
import com.huimv.farm.damsubsidy.entity.vo.BillLandingImageVo;
import com.huimv.farm.damsubsidy.mapper.*;
import com.huimv.farm.damsubsidy.service.IBillLandingInspectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillLandingInspectionServiceImpl extends ServiceImpl<BillLandingInspectionMapper, BillLandingInspection> implements IBillLandingInspectionService {

    @Resource
    private BillLandingInspectionMapper billLandingInspectionMapper;
    @Resource
    private BillIsolatedMapper isolatedMapper;
    @Resource
    private BillBuyInfoMapper buyInfoMapper;
    @Resource
    private BillSubsidyMapper subsidyMapper;
    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private BaseAnimalMapper baseAnimalMapper;

    @Autowired
    private UploadImage uploadImage;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        Integer type = Integer.parseInt(paramsMap.get("type"));//信息类型
        String date = paramsMap.get("data");//状态
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageNum");
        if (pageSize == null || pageSize == "") {
            pageSize = "10";
        }
        if (pageNum == null || pageNum == "") {
            pageNum = "1";
        }

        System.out.println(type);
        if (type == 1) { //检疫申请
            QueryWrapper<BillLandingInspection> queryWrapper = new QueryWrapper<>();
            if (date == null || date == "") {
                queryWrapper.orderByDesc("create_time");
            } else {
                queryWrapper.orderByDesc("create_time").eq("inspection_sch", date);
            }
            Page<BillLandingInspection> billLandingInspectionPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, billLandingInspectionMapper.selectPage(billLandingInspectionPage, queryWrapper));
        }
        //交易信息
        if (type == 2) {
            QueryWrapper<BillBuyInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillBuyInfo> buyInfoPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, buyInfoMapper.selectPage(buyInfoPage, queryWrapper));
        }
        //补贴申请
        if (type == 3) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        //验收信息
        if (type == 4) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        //支付信息
        if (type == 5) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time").eq("subsidy_sch", 3);
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        return new Result(ResultCode.FAIL);
    }

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        Integer id = Integer.valueOf(paramsMap.get("id"));
        String num = paramsMap.get("num");//检疫证号
        Integer type = Integer.parseInt(paramsMap.get("type"));//消息类型
        JSONObject jsonObject = new JSONObject();
        if (type == 1) {
            //检疫
            jsonObject.put("data1", billLandingInspectionMapper.selectById(id));
            jsonObject.put("data2", isolatedMapper.selectOne(new QueryWrapper<BillIsolated>().eq("inspection_num", num)));
        }
        if (type == 2) {
            //交易
            jsonObject.put("data1", buyInfoMapper.selectById(id));
            jsonObject.put("data2", billLandingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", num)));
        }
        if (type == 3) {
            //补贴
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        if (type == 4) {
            //验收
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        if (type == 5) {
            //支付
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        return new Result(ResultCode.SUCCESS, jsonObject);

    }

    @Override
    public Result addPay(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String url = paramsMap.get("url");
        BillSubsidy billSubsidy = subsidyMapper.selectById(id);
        billSubsidy.setPayImgUrl(url);
        billSubsidy.setSubsidySch(3);
        billSubsidy.setIsPay(2);
        billSubsidy.setPayMoney(billSubsidy.getApplyMoney());
        subsidyMapper.updateById(billSubsidy);
        return new Result(ResultCode.SUCCESS,subsidyMapper.selectById(id));
    }

    @Override
    public Result addApplication(HttpServletRequest httpServletRequest, String inspectionNum, String cargoOwner, String ownerPhone, String animalType, String animalNumberUp, String useTo, String startTran, String reachTran, String veter, String eartags, MultipartFile quarantineImage, MultipartFile image1,MultipartFile image2,MultipartFile image3,MultipartFile image4,MultipartFile image5) throws IOException {
        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        landingInspectionQueryWrapper.eq("inspection_num", inspectionNum).ne("inspection_sch",5);
        if (this.count(landingInspectionQueryWrapper) != 0) {
            return new Result(10001, "存在相同检疫证号！", false);
        }
        if (inspectionNum.length() != 10) {
            return new Result(10001, "检疫证号应该为10位！", false);
        }
        if (!IntegerUtil.isInteger(animalNumberUp)){
            return new Result(10001, "头数应为整数！", false);
        }
       String content = "";
       int total = Integer.parseInt(animalNumberUp);

        EartagResult eartagResult = EartagUtil.getEartagCollection(eartags);
        if (!eartagResult.getSuccess()){
            return new Result(10001,eartagResult.getMsg(),false);
        }
        if (eartagResult.getEartagList().size() != eartagResult.getEartagSet().size()) {
            return new Result(10001,"耳标号存在重复",false);
        }
        System.out.println("具体耳标号"+eartagResult.getEartagList());
        if (eartagResult.getEartagList().size() != total){
            return new Result(10001,"数量与实际耳标号数量不符!",false);
        }
        content = getUrl(content,image1);
        content = getUrl(content,image2);
        content = getUrl(content,image3);
        content = getUrl(content,image4);
        content = getUrl(content,image5);
        String imageCom = "";
        if (ObjectUtil.isNotEmpty(quarantineImage)){
            imageCom = uploadImage.getImageCom(quarantineImage);
        }
        BillLandingInspection billLandingInspection = new BillLandingInspection();
        billLandingInspection.setInspectionNum(inspectionNum);
        billLandingInspection.setCargoOwner(cargoOwner);
        billLandingInspection.setOwnerPhone(ownerPhone);
        billLandingInspection.setAnimalType(Integer.parseInt(animalType));
        billLandingInspection.setAnimalNumberUp(animalNumberUp);
        billLandingInspection.setUseTo(useTo);
        billLandingInspection.setStartTran(startTran);
        billLandingInspection.setReachTran(reachTran);
        billLandingInspection.setVeterId(Integer.parseInt(veter));
        billLandingInspection.setQuarantineCertificatePic(imageCom);
        billLandingInspection.setEartags(eartags);
        billLandingInspection.setJcbgImgUrl(content);
        billLandingInspection.setInspectionName(TokenSign.getUserName(httpServletRequest) + "的落地检疫申请");
        billLandingInspection.setCreateTime(DateTime.now());
        billLandingInspection.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
        billLandingInspection.setInspectionType(0);
        if (this.save(billLandingInspection)) {
            return Result.SUCCESS();
        } else {
            return Result.FAIL();
        }
    }

    @Override
    public Result listReadyApplication() {
        List<BillLandingInspection> inspectionNumList = billLandingInspectionMapper.listReadyApplication();
        if (ObjectUtil.isNotEmpty(inspectionNumList)){
            return new Result(ResultCode.SUCCESS,inspectionNumList);
        }else {
            return new Result(10001,"无符合要求的检疫证号!",false);
        }
    }


    public  String getUrl(String content, MultipartFile image) throws IOException {
        if (ObjectUtil.isNotEmpty(image)){
            if ( !"".equals(content)){
                content = content +","+uploadImage.getImageCom(image);
            }else {
                content =   content +uploadImage.getImageCom(image);
            }
        }
        return content;
    }

    @Override
    public Result cancelApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        if (this.removeById(id)) {
            return Result.SUCCESS();
        }else {
            return Result.ERROR();
        }
    }

    @Override
    public Result listApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageNum");
        String inspectionSch = paramsMap.get("inspectionSch");
        Long userName = TokenSign.getMemberIdByJwtToken(httpServletRequest);
        SysUser user = sysUserMapper.selectOne(new QueryWrapper<SysUser>().eq("id", userName));
        Integer userType = user.getUserType();

        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        if (userType == 2 || userType == 3){
            landingInspectionQueryWrapper.eq("create_user",userName);
        }else if (userType == 1){
            landingInspectionQueryWrapper.eq("veter_id",user.getVeterId()).eq("inspection_type",0);
        }
        if ("4".equals(inspectionSch)){
            landingInspectionQueryWrapper.in("inspection_sch",4,5);
        } else if (ObjectUtil.isNotEmpty(inspectionSch)){
            landingInspectionQueryWrapper.eq("inspection_sch",inspectionSch);
        }else if(ObjectUtil.isEmpty(inspectionSch)){
            landingInspectionQueryWrapper.ne("inspection_sch",6);
        }
        landingInspectionQueryWrapper.orderByDesc("create_time");
        Page<BillLandingInspection> page = new Page(Integer.parseInt(pageNum),Integer.parseInt(pageSize));
        Page<BillLandingInspection> billLandingInspectionPage = billLandingInspectionMapper.selectPage(page, landingInspectionQueryWrapper);
        Page<BillLandingImageVo> billLandingImageVoPage = new Page<>();

        BeanUtil.copyProperties(billLandingInspectionPage,billLandingImageVoPage);
        List billLandingImageVoList = new ArrayList<>();
        for (BillLandingInspection record : billLandingInspectionPage.getRecords()) {
            BillLandingImageVo billLandingImageVo = new BillLandingImageVo();
            BeanUtil.copyProperties(record,billLandingImageVo);
            if (ObjectUtil.isNotEmpty(record.getJcbgImgUrl())){
                billLandingImageVo.setPages(record.getJcbgImgUrl().split(","));
            }
            billLandingImageVoList.add(billLandingImageVo);
        }
        billLandingImageVoPage.setRecords(billLandingImageVoList);
       /* billLandingImageVoPage.setRecords(billLandingImageVoList);
        BeanUtil.copyProperties(billLandingInspectionPage,billLandingImageVoPage);
        for (BillLandingImageVo record : billLandingImageVoPage.getRecords()) {
            if (ObjectUtil.isNotEmpty(record.getJcbgImgUrl())){
                record.setPages(record.getJcbgImgUrl().split(","));
            }
        }*/
        return new Result(ResultCode.SUCCESS, billLandingImageVoPage);
    }

    @Override
    public Result rejectApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String rejectReason = paramsMap.get("rejectReason");
        BillLandingInspection billLandingInspection = this.getById(id);
        billLandingInspection.setInspectionSch(5);
        billLandingInspection.setRejectReason(rejectReason);
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setReviewedTime(DateTime.now());
        this.updateById(billLandingInspection);
        return Result.SUCCESS();
    }

    @Override
    public Result agreeApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BillLandingInspection landingInspection = this.getById(id);
        landingInspection.setInspectionSch(2);
        landingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        landingInspection.setReviewedTime(DateTime.now());
        if (this.updateById(landingInspection)){
            return Result.SUCCESS();
        }else {
            return Result.ERROR();
        }
    }

    @Override
    public Result getEartagNoBy(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String inspectionNum = paramsMap.get("inspectionNum");
        List<BaseAnimal> baseAnimals = baseAnimalMapper.selectList(new QueryWrapper<BaseAnimal>().eq("inspection_num",inspectionNum));
        if (ObjectUtil.isEmpty(baseAnimals)){
            return new Result(10001,"当前检疫无耳标",false);
        }
        return new Result(ResultCode.SUCCESS,baseAnimals);
    }
}
